@@ -158,7 +158,7 @@ xplr.config.general.logs.error.style = { fg = "Red" }
 -- * style: [Style](https://xplr.dev/en/style)
 xplr.config.general.table.header.cols = {
   { format = " index", style = {} },
-  { format = "╭──── path", style = {} },
+  { format = "╭─── path", style = {} },
   { format = "permissions", style = {} },
   { format = "size", style = {} },
   { format = "modified", style = {} },
@@ -225,9 +225,9 @@ xplr.config.general.table.style = {}
 -- * format: nullable string
 -- * style: [Style](https://xplr.dev/en/style)
 xplr.config.general.table.tree = {
-  { format = "├─", style = {} },
-  { format = "├─", style = {} },
-  { format = "╰─", style = {} },
+  { format = "├", style = {} },
+  { format = "├", style = {} },
+  { format = "╰", style = {} },
 }
 
 -- Spacing between the columns in the table.
@@ -249,7 +249,7 @@ xplr.config.general.table.col_widths = {
 -- The content that is placed before the item name for each row by default.
 --
 -- Type: nullable string
-xplr.config.general.default_ui.prefix = "  "
+xplr.config.general.default_ui.prefix = "─ "
 
 -- The content which is appended to each item name for each row by default.
 --
@@ -278,7 +278,7 @@ xplr.config.general.focus_ui.style = { add_modifiers = { "Bold" } }
 -- The string placed before the item name for a selected row.
 --
 -- Type: nullable string
-xplr.config.general.selection_ui.prefix = " {"
+xplr.config.general.selection_ui.prefix = "─{"
 
 -- The string placed after the item name for a selected row.
 --
@@ -336,26 +336,22 @@ xplr.config.general.sort_and_filter_ui.default_identifier.style = {}
 -- The shape of the forward direction indicator for sort identifiers in Sort & filter panel.
 --
 -- Type: nullable string
-xplr.config.general.sort_and_filter_ui.sort_direction_identifiers.forward.format =
-  "↓"
+xplr.config.general.sort_and_filter_ui.sort_direction_identifiers.forward.format = "↓"
 
 -- Style of forward direction indicator in Sort & filter panel.
 --
 -- Type: [Style](https://xplr.dev/en/style)
-xplr.config.general.sort_and_filter_ui.sort_direction_identifiers.forward.style =
-  nil
+xplr.config.general.sort_and_filter_ui.sort_direction_identifiers.forward.style = nil
 
 -- The shape of the reverse direction indicator for sort identifiers in Sort & filter panel.
 --
 -- Type: nullable string
-xplr.config.general.sort_and_filter_ui.sort_direction_identifiers.reverse.format =
-  "↑"
+xplr.config.general.sort_and_filter_ui.sort_direction_identifiers.reverse.format = "↑"
 
 -- Style of reverse direction indicator in Sort & filter panel.
 --
 -- Type: [Style](https://xplr.dev/en/style)
-xplr.config.general.sort_and_filter_ui.sort_direction_identifiers.reverse.style =
-  nil
+xplr.config.general.sort_and_filter_ui.sort_direction_identifiers.reverse.style = nil
 
 -- The identifiers used to denote applied sorters in the Sort & filter panel.
 --
@@ -664,6 +660,25 @@ xplr.config.general.initial_layout = "default"
 -- Type: nullable string
 xplr.config.general.start_fifo = nil
 
+-- Use it to define a set of key bindings that are available by default in
+-- every [mode](https://xplr.dev/en/mode). They can be overwritten.
+--
+-- Type: [Key Bindings](https://xplr.dev/en/configure-key-bindings#key-bindings)
+xplr.config.general.global_key_bindings = {
+  on_key = {
+    esc = {
+      messages = {
+        "PopMode",
+      },
+    },
+    ["ctrl-c"] = {
+      messages = {
+        "Terminate",
+      },
+    },
+  },
+}
+
 -- ### Node Types -------------------------------------------------------------
 --
 -- This section defines how to deal with different kinds of nodes (files,
@@ -683,22 +698,38 @@ xplr.config.node_types.directory.style = {
   add_modifiers = { "Bold" },
 }
 
--- Metadata for the directory nodes
+-- Metadata for the directory nodes.
+-- You can set as many metadata as you want.
 --
 -- Type: nullable string
+--
+-- Example:
+--
+-- ```lua
+-- xplr.config.node_types.directory.meta.foo = "foo"
+-- xplr.config.node_types.directory.meta.bar = "bar"
+-- ```
 xplr.config.node_types.directory.meta.icon = "ð"
 
--- The style for the file nodes
+-- The style for the file nodes.
 --
 -- Type: [Style](https://xplr.dev/en/style)
 xplr.config.node_types.file.style = {}
 
--- Metadata for the file nodes
+-- Metadata for the file nodes.
+-- You can set as many metadata as you want.
 --
 -- Type: nullable string
+--
+-- Example:
+--
+-- ```lua
+-- xplr.config.node_types.file.meta.foo = "foo"
+-- xplr.config.node_types.file.meta.bar = "bar"
+-- ```
 xplr.config.node_types.file.meta.icon = "ƒ"
 
--- The style for the symlink nodes
+-- The style for the symlink nodes.
 --
 -- Type: [Style](https://xplr.dev/en/style)
 xplr.config.node_types.symlink.style = {
@@ -706,9 +737,17 @@ xplr.config.node_types.symlink.style = {
   add_modifiers = { "Italic" },
 }
 
--- Metadata for the symlink nodes
+-- Metadata for the symlink nodes.
+-- You can set as many metadata as you want.
 --
 -- Type: nullable string
+--
+-- Example:
+--
+-- ```lua
+-- xplr.config.node_types.symlink.meta.foo = "foo"
+-- xplr.config.node_types.symlink.meta.bar = "bar"
+-- ```
 xplr.config.node_types.symlink.meta.icon = "§"
 
 -- Metadata and style based on mime types.
@@ -732,13 +771,12 @@ xplr.config.node_types.symlink.meta.icon = "§"
 --
 --     -- application/pdf
 --     pdf = { meta = { icon = "" }, style = { fg = "Blue" } },
-
+--
 --     -- application/zip
 --     zip = { meta = { icon = ""} },
 --   },
 -- }
 -- ```
-
 xplr.config.node_types.mime_essence = {}
 
 -- Metadata and style based on extension.
@@ -1022,12 +1060,6 @@ xplr.config.modes.builtin.default = {
           "ToggleSelectAll",
         },
       },
-      ["ctrl-c"] = {
-        help = "terminate",
-        messages = {
-          "Terminate",
-        },
-      },
       ["ctrl-f"] = {
         help = "search",
         messages = {
@@ -1087,10 +1119,6 @@ xplr.config.modes.builtin.default = {
           "PrintResultAndQuit",
         },
       },
-      esc = {
-        help = nil,
-        messages = {},
-      },
       ["f"] = {
         help = "filter",
         messages = {
@@ -1124,7 +1152,8 @@ xplr.config.modes.builtin.default = {
           { SwitchModeBuiltin = "rename" },
           {
             BashExecSilently = [===[
-              echo SetInputBuffer: "'"$(basename "${XPLR_FOCUS_PATH}")"'" >> "${XPLR_PIPE_MSG_IN:?}"
+              NAME=$(basename "${XPLR_FOCUS_PATH:?}")
+              echo SetInputBuffer: "'"${NAME:?}"'" >> "${XPLR_PIPE_MSG_IN:?}"
             ]===],
           },
         },
@@ -1136,7 +1165,8 @@ xplr.config.modes.builtin.default = {
           { SwitchModeBuiltin = "duplicate_as" },
           {
             BashExecSilently = [===[
-              echo SetInputBuffer: "'"$(basename "${XPLR_FOCUS_PATH}")"'" >> "${XPLR_PIPE_MSG_IN:?}"
+              NAME=$(basename "${XPLR_FOCUS_PATH:?}")
+              echo SetInputBuffer: "'"${NAME:?}"'" >> "${XPLR_PIPE_MSG_IN:?}"
             ]===],
           },
         },
@@ -1183,6 +1213,7 @@ xplr.config.modes.builtin.default = {
       messages = {
         "PopMode",
         { SwitchModeBuiltin = "number" },
+        { SetInputPrompt = ":" },
         "BufferInputFromKey",
       },
     },
@@ -1255,12 +1286,6 @@ xplr.config.modes.builtin.debug_error = {
   },
   key_bindings = {
     on_key = {
-      ["ctrl-c"] = {
-        help = "terminate",
-        messages = {
-          "Terminate",
-        },
-      },
       enter = {
         help = "open logs in editor",
         messages = {
@@ -1271,12 +1296,6 @@ xplr.config.modes.builtin.debug_error = {
           },
         },
       },
-      esc = {
-        help = "escape",
-        messages = {
-          "PopMode",
-        },
-      },
       q = {
         help = "quit",
         messages = {
@@ -1314,23 +1333,46 @@ xplr.config.modes.builtin.recover = {
       },
     },
   },
+  key_bindings = {
+    default = {
+      messages = {},
+    },
+  },
+}
+
+-- The builtin go to path mode.
+--
+-- Type: [Mode](https://xplr.dev/en/mode)
+xplr.config.modes.builtin.go_to_path = {
+  name = "go to path",
   key_bindings = {
     on_key = {
-      ["ctrl-c"] = {
-        help = "terminate",
+      enter = {
+        help = "submit",
         messages = {
-          "Terminate",
+          {
+            BashExecSilently = [===[
+              if [ -d "$XPLR_INPUT_BUFFER" ]; then
+                echo ChangeDirectory: "'"$XPLR_INPUT_BUFFER"'" >> "${XPLR_PIPE_MSG_IN:?}"
+              elif [ -e "$XPLR_INPUT_BUFFER" ]; then
+                echo FocusPath: "'"$XPLR_INPUT_BUFFER"'" >> "${XPLR_PIPE_MSG_IN:?}"
+              fi
+            ]===],
+          },
+          "PopMode",
         },
       },
-      esc = {
-        help = "escape",
+      tab = {
+        help = "try complete",
         messages = {
-          "PopMode",
+          { CallLuaSilently = "builtin.try_complete_path" },
         },
       },
     },
     default = {
-      messages = {},
+      messages = {
+        "UpdateInputBufferFromKey",
+      },
     },
   },
 }
@@ -1362,18 +1404,6 @@ xplr.config.modes.builtin.selection_ops = {
           "PopMode",
         },
       },
-      ["ctrl-c"] = {
-        help = "terminate",
-        messages = {
-          "Terminate",
-        },
-      },
-      esc = {
-        help = "cancel",
-        messages = {
-          "PopMode",
-        },
-      },
       ["m"] = {
         help = "move here",
         messages = {
@@ -1428,31 +1458,21 @@ xplr.config.modes.builtin.create = {
   name = "create",
   key_bindings = {
     on_key = {
-      ["ctrl-c"] = {
-        help = "terminate",
-        messages = {
-          "Terminate",
-        },
-      },
-      ["d"] = {
+      d = {
         help = "create directory",
         messages = {
           "PopMode",
-          { SwitchModeBuiltin = "create directory" },
+          { SwitchModeBuiltin = "create_directory" },
+          { SetInputPrompt = "ð ❯ " },
           { SetInputBuffer = "" },
         },
       },
-      esc = {
-        help = "cancel",
-        messages = {
-          "PopMode",
-        },
-      },
-      ["f"] = {
+      f = {
         help = "create file",
         messages = {
           "PopMode",
-          { SwitchModeBuiltin = "create file" },
+          { SwitchModeBuiltin = "create_file" },
+          { SetInputPrompt = "ƒ ❯ " },
           { SetInputBuffer = "" },
         },
       },
@@ -1467,14 +1487,14 @@ xplr.config.modes.builtin.create_directory = {
   name = "create directory",
   key_bindings = {
     on_key = {
-      ["ctrl-c"] = {
-        help = "terminate",
+      tab = {
+        help = "try complete",
         messages = {
-          "Terminate",
+          { CallLuaSilently = "builtin.try_complete_path" },
         },
       },
       enter = {
-        help = "create directory",
+        help = "submit",
         messages = {
           {
             BashExecSilently = [===[
@@ -1492,12 +1512,6 @@ xplr.config.modes.builtin.create_directory = {
           },
         },
       },
-      esc = {
-        help = "cancel",
-        messages = {
-          "PopMode",
-        },
-      },
     },
     default = {
       messages = {
@@ -1514,12 +1528,14 @@ xplr.config.modes.builtin.create_file = {
   name = "create file",
   key_bindings = {
     on_key = {
-      ["ctrl-c"] = {
-        help = "terminate",
-        messages = { "Terminate" },
+      tab = {
+        help = "try complete",
+        messages = {
+          { CallLuaSilently = "builtin.try_complete_path" },
+        },
       },
       enter = {
-        help = "create file",
+        help = "submit",
         messages = {
           {
             BashExecSilently = [===[
@@ -1538,12 +1554,6 @@ xplr.config.modes.builtin.create_file = {
           },
         },
       },
-      esc = {
-        help = "cancel",
-        messages = {
-          "PopMode",
-        },
-      },
     },
     default = {
       messages = {
@@ -1560,12 +1570,6 @@ xplr.config.modes.builtin.number = {
   name = "number",
   key_bindings = {
     on_key = {
-      ["ctrl-c"] = {
-        help = "terminate",
-        messages = {
-          "Terminate",
-        },
-      },
       down = {
         help = "to down",
         messages = {
@@ -1580,12 +1584,6 @@ xplr.config.modes.builtin.number = {
           "PopMode",
         },
       },
-      esc = {
-        help = "cancel",
-        messages = {
-          "PopMode",
-        },
-      },
       up = {
         help = "to up",
         messages = {
@@ -1620,33 +1618,29 @@ xplr.config.modes.builtin.go_to = {
   name = "go to",
   key_bindings = {
     on_key = {
-      ["ctrl-c"] = {
-        help = "terminate",
-        messages = {
-          "Terminate",
-        },
-      },
-      esc = {
-        help = "cancel",
-        messages = {
-          "PopMode",
-        },
-      },
-      ["f"] = {
+      f = {
         help = "follow symlink",
         messages = {
           "FollowSymlink",
           "PopMode",
         },
       },
-      ["g"] = {
+      g = {
         help = "top",
         messages = {
           "FocusFirst",
           "PopMode",
         },
       },
-      ["x"] = {
+      p = {
+        help = "path",
+        messages = {
+          "PopMode",
+          { SwitchModeBuiltin = "go_to_path" },
+          { SetInputBuffer = "" },
+        },
+      },
+      x = {
         help = "open in gui",
         messages = {
           {
@@ -1679,14 +1673,14 @@ xplr.config.modes.builtin.rename = {
   name = "rename",
   key_bindings = {
     on_key = {
-      ["ctrl-c"] = {
-        help = "terminate",
+      tab = {
+        help = "try complete",
         messages = {
-          "Terminate",
+          { CallLuaSilently = "builtin.try_complete_path" },
         },
       },
       enter = {
-        help = "rename",
+        help = "submit",
         messages = {
           {
             BashExecSilently = [===[
@@ -1705,12 +1699,6 @@ xplr.config.modes.builtin.rename = {
           "PopMode",
         },
       },
-      esc = {
-        help = "cancel",
-        messages = {
-          "PopMode",
-        },
-      },
     },
     default = {
       messages = {
@@ -1727,14 +1715,14 @@ xplr.config.modes.builtin.duplicate_as = {
   name = "duplicate as",
   key_bindings = {
     on_key = {
-      ["ctrl-c"] = {
-        help = "terminate",
+      tab = {
+        help = "try complete",
         messages = {
-          "Terminate",
+          { CallLuaSilently = "builtin.try_complete_path" },
         },
       },
       enter = {
-        help = "duplicate",
+        help = "submit",
         messages = {
           {
             BashExecSilently = [===[
@@ -1753,12 +1741,6 @@ xplr.config.modes.builtin.duplicate_as = {
           "PopMode",
         },
       },
-      esc = {
-        help = "cancel",
-        messages = {
-          "PopMode",
-        },
-      },
     },
     default = {
       messages = {
@@ -1794,12 +1776,6 @@ xplr.config.modes.builtin.delete = {
           "PopMode",
         },
       },
-      ["ctrl-c"] = {
-        help = "terminate",
-        messages = {
-          "Terminate",
-        },
-      },
       ["d"] = {
         help = "delete",
         messages = {
@@ -1827,12 +1803,6 @@ xplr.config.modes.builtin.delete = {
           "PopMode",
         },
       },
-      esc = {
-        help = "cancel",
-        messages = {
-          "PopMode",
-        },
-      },
     },
   },
 }
@@ -1859,12 +1829,6 @@ xplr.config.modes.builtin.action = {
           { SwitchModeBuiltin = "create" },
         },
       },
-      ["ctrl-c"] = {
-        help = "terminate",
-        messages = {
-          "Terminate",
-        },
-      },
       ["e"] = {
         help = "open in editor",
         messages = {
@@ -1876,12 +1840,6 @@ xplr.config.modes.builtin.action = {
           "PopMode",
         },
       },
-      esc = {
-        help = "cancel",
-        messages = {
-          "PopMode",
-        },
-      },
       ["l"] = {
         help = "logs",
         messages = {
@@ -1921,6 +1879,7 @@ xplr.config.modes.builtin.action = {
       messages = {
         "PopMode",
         { SwitchModeBuiltin = "number" },
+        { SetInputPrompt = ":" },
         "BufferInputFromKey",
       },
     },
@@ -1964,18 +1923,6 @@ xplr.config.modes.builtin.quit = {
           "PrintResultAndQuit",
         },
       },
-      esc = {
-        help = "cancel",
-        messages = {
-          "PopMode",
-        },
-      },
-      ["ctrl-c"] = {
-        help = "terminate",
-        messages = {
-          "Terminate",
-        },
-      },
     },
   },
 }
@@ -1987,12 +1934,6 @@ xplr.config.modes.builtin.search = {
   name = "search",
   key_bindings = {
     on_key = {
-      ["ctrl-c"] = {
-        help = "terminate",
-        messages = {
-          "Terminate",
-        },
-      },
       down = {
         help = "down",
         messages = {
@@ -2000,7 +1941,7 @@ xplr.config.modes.builtin.search = {
         },
       },
       enter = {
-        help = "focus",
+        help = "submit",
         messages = {
           { RemoveNodeFilterFromInput = "RelativePathDoesMatchRegex" },
           "PopMode",
@@ -2088,12 +2029,6 @@ xplr.config.modes.builtin.filter = {
           "ExplorePwdAsync",
         },
       },
-      enter = {
-        help = "done",
-        messages = {
-          "PopMode",
-        },
-      },
       backspace = {
         help = "remove last filter",
         messages = {
@@ -2115,19 +2050,10 @@ xplr.config.modes.builtin.filter = {
           "ExplorePwdAsync",
         },
       },
-      ["ctrl-c"] = {
-        help = "terminate",
-        messages = {
-          "Terminate",
-        },
-      },
     },
   },
 }
 
-xplr.config.modes.builtin.filter.key_bindings.on_key["esc"] =
-  xplr.config.modes.builtin.filter.key_bindings.on_key.enter
-
 -- The builtin relative_path_does_match_regex mode.
 --
 -- Type: [Mode](https://xplr.dev/en/mode)
@@ -2135,20 +2061,13 @@ xplr.config.modes.builtin.relative_path_does_match_regex = {
   name = "relative path does match regex",
   key_bindings = {
     on_key = {
-      ["ctrl-c"] = {
-        help = "terminate",
-        messages = {
-          "Terminate",
-        },
-      },
       enter = {
-        help = "apply filter",
+        help = "submit",
         messages = {
           "PopMode",
         },
       },
       esc = {
-        help = "cancel",
         messages = {
           { RemoveNodeFilterFromInput = "RelativePathDoesMatchRegex" },
           "PopMode",
@@ -2174,20 +2093,13 @@ xplr.config.modes.builtin.relative_path_does_not_match_regex = {
   name = "relative path does not match regex",
   key_bindings = {
     on_key = {
-      ["ctrl-c"] = {
-        help = "terminate",
-        messages = {
-          "Terminate",
-        },
-      },
       enter = {
-        help = "apply filter",
+        help = "submit",
         messages = {
           "PopMode",
         },
       },
       esc = {
-        help = "cancel",
         messages = {
           { RemoveNodeFilterFromInput = "RelativePathDoesNotMatchRegex" },
           "PopMode",
@@ -2299,12 +2211,6 @@ xplr.config.modes.builtin.sort = {
           "ExplorePwdAsync",
         },
       },
-      ["ctrl-c"] = {
-        help = "terminate",
-        messages = {
-          "Terminate",
-        },
-      },
       ["ctrl-r"] = {
         help = "reset sorters",
         messages = {
@@ -2332,7 +2238,7 @@ xplr.config.modes.builtin.sort = {
         },
       },
       enter = {
-        help = "done",
+        help = "submit",
         messages = {
           "PopMode",
         },
@@ -2414,9 +2320,6 @@ xplr.config.modes.builtin.sort = {
   },
 }
 
-xplr.config.modes.builtin.sort.key_bindings.on_key["esc"] =
-  xplr.config.modes.builtin.sort.key_bindings.on_key.enter
-
 -- The builtin switch layout mode.
 --
 -- Type: [Mode](https://xplr.dev/en/mode)
@@ -2452,18 +2355,6 @@ xplr.config.modes.builtin.switch_layout = {
           "PopMode",
         },
       },
-      ["ctrl-c"] = {
-        help = "terminate",
-        messages = {
-          "Terminate",
-        },
-      },
-      esc = {
-        help = "cancel",
-        messages = {
-          "PopMode",
-        },
-      },
     },
   },
 }
@@ -2507,6 +2398,62 @@ xplr.config.modes.custom = {}
 -- As always, `xplr.fn.builtin` is where the built-in functions are defined
 -- that can be overwritten.
 
+-- Tries to auto complete the path in the input buffer
+xplr.fn.builtin.try_complete_path = function(m)
+  if not m.input_buffer then
+    return
+  end
+
+  local function splitlines(str)
+    local res = {}
+    for s in str:gmatch("[^\r\n]+") do
+      table.insert(res, s)
+    end
+    return res
+  end
+
+  local function matches_all(str, files)
+    for _, p in ipairs(files) do
+      if string.sub(p, 1, #str) ~= str then
+        return false
+      end
+    end
+    return true
+  end
+
+  local path = m.input_buffer
+
+  local p = assert(io.popen(string.format("ls -d %q* 2>/dev/null", path)))
+  local out = p:read("*all")
+  p:close()
+
+  local found = splitlines(out)
+  local count = #found
+
+  if count == 0 then
+    return
+  elseif count == 1 then
+    return {
+      { SetInputBuffer = found[1] },
+    }
+  else
+    local first = found[1]
+    while #first > #path and matches_all(path, found) do
+      path = string.sub(found[1], 1, #path + 1)
+    end
+
+    if matches_all(path, found) then
+      return {
+        { SetInputBuffer = path },
+      }
+    end
+
+    return {
+      { SetInputBuffer = string.sub(path, 1, #path - 1) },
+    }
+  end
+end
+
 -- Renders the first column in the table
 xplr.fn.builtin.fmt_general_table_row_cols_0 = function(m)
   local r = ""
